import bpy
from bpy.props import StringProperty, EnumProperty
from bpy.types import AddonPreferences

from . import parameters
from . import utils

def update_welder_category(self, context):
    try:
        bpy.utils.unregister_class(PANEL_PT_WelderToolsPanel)
        bpy.utils.unregister_class(PANEL_PT_WelderSubPanelDynamic)
    except:
        pass
    PANEL_PT_WelderToolsPanel.bl_category = context.preferences.addons[parameters.NAME].preferences.category
    PANEL_PT_WelderSubPanelDynamic.bl_category = context.preferences.addons[parameters.NAME].preferences.category
    bpy.utils.register_class(PANEL_PT_WelderToolsPanel)
    bpy.utils.register_class(PANEL_PT_WelderSubPanelDynamic) 

class PANEL_PT_WelderToolsPanel(bpy.types.Panel):
    bl_label = parameters.NAME.rsplit(".")[-1]
    bl_idname = "OBJECT_PT_Welder"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = parameters.NAME.rsplit(".")[-1]
 
    active=False 
    weld=None
 
    @classmethod
    def poll(self, context):
        if (context.active_object != None):
            if bpy.context.view_layer.objects.active.get('Weld') is not None:         
                self.weld=bpy.context.view_layer.objects.active           
                self.active= True
            else:
                self.active= False
        else:
            self.active= True            
        return True        
 
    def draw(self, context):
        
        if (self.active):
            try:
                #row=self.layout.row()
                #row.template_icon_view(self.weld.weld, "thumbnails") #add interactive and updatemethod
                row=self.layout.row()
                #row.prop(self.weld.weld,"name")
                row=self.layout.row()
                row.prop(utils.getSpline(),"use_cyclic_u",text="循环")
                collapsebox = self.layout.box()   
                row = collapsebox.row(align=True)
                row.label(text="Collapse")
                row = collapsebox.row()
                row.prop(context.scene, "collapsesubsurf")
                row = collapsebox.row()
                row.operator("weld.collapse")
                #row.prop(self.weld.weld, "blend")
            except:
                pass    
            '''
            row.template_icon_view(context.scene, "my_thumbnails")
            row.enabled=not bpy.context.scene.welddrawing
            row=self.layout.row()
            row.operator("weld.weld")
            row.enabled=not bpy.context.scene.welddrawing
            row=self.layout.row()
            row.operator("weld.draw")
            row.enabled=not bpy.context.scene.welddrawing
            row=self.layout.row()
            
            row.prop(context.scene, "surfaceblend")
            row=self.layout.row()
            row.prop(context.scene, 'type', expand=True) 
            ''' 
        else:
            row=self.layout.row()
            row.template_icon_view(context.scene, "my_thumbnails")
            row.enabled=not bpy.context.scene.welddrawing
            row=self.layout.row()
            row.operator("weld.weld")
            row.enabled=not bpy.context.scene.welddrawing
            row=self.layout.row()
            row.operator("weld.draw")
            row.enabled=not bpy.context.scene.welddrawing
            row=self.layout.row()
            row.prop(context.scene, "cyclic")
            row.prop(context.scene, "surfaceblend")
            row=self.layout.row()
            row.prop(context.scene, 'type', expand=True)          
        
class PANEL_PT_WelderSubPanelDynamic(bpy.types.Panel):        
    bl_label = "Shape"
    bl_idname = "OBJECT_PT_Welder_dynamic"
    bl_space_type = "VIEW_3D"
    bl_region_type = "UI"
    bl_category = parameters.NAME

    
    @classmethod
    def poll(cls, context):
        if (context.active_object != None):
            if bpy.context.view_layer.objects.active.get('Weld') is not None:                
                return True
            else: return False
        else: return False
    
    def draw(self, context):  
        #bpy.context.scene.cyclic=cehckIfCyclic()
        row=self.layout.row()
        row.operator("weld.shape", text=bpy.context.scene.shapebuttonname)
        box = self.layout.box() 
        box.enabled= bpy.context.scene.shapemodified 
        box.row()
        if bpy.context.scene.shapemodified: box.template_curve_mapping(utils.WeldCurveData('WeldCurve',self), "mapping")   
        else: utils.removenode()
        row=self.layout.row()
        row.operator("weld.optimize")     

class WelderPreferences(bpy.types.AddonPreferences):
    bl_idname = parameters.NAME
    prefs_tabs: EnumProperty(
    items=(('info', "Info", "Welder Info"),
           ('options', "Options", "Welder Options")),
    default='info')
    
    category : StringProperty(description="选择面板类别的名称",default=parameters.NAME.rsplit(".")[-1], update=update_welder_category)
    performance : EnumProperty(
    items=(('Fast', "Fast", "Fast"),
           ('High quality', "High quality", "High quality")),
    default='Fast')
    solver: EnumProperty(
    items=(('FAST', "FAST", "FAST"),
           ('EXACT', "EXACT", "EXACT")),
    default='FAST')

    def draw(self, context):
        wm = context.window_manager
        layout = self.layout

        row= layout.row(align=True)
        row.prop(self, "prefs_tabs", expand=True)
        if self.prefs_tabs == 'info':
            layout.label(text="1.沿交点焊接")
            layout.label(text="-在对象模式下选择2个对象，")
            layout.label(text="-选择要放置的焊缝类型 (几何/贴花)")
            layout.label(text="-单击 “焊接” 按钮")
            layout.label(text="-通过移动鼠标调整比例 (鼠标右键应用，设置为默认比例)")
            layout.label(text="-通过移动鼠标调整旋转 (鼠标右键应用，设置为默认旋转)")
            layout.label(text="2.沿选定的边缘进行焊接")
            layout.label(text="-在编辑模式下选择所需的edgeloop，")
            layout.label(text="-选择要放置的焊缝类型 (几何/贴花)")
            layout.label(text="-单击 “焊接” 按钮")
            layout.label(text="-通过移动鼠标调整比例 (鼠标右键应用，设置为默认比例)")
            layout.label(text="-通过移动鼠标调整旋转 (鼠标右键应用，设置为默认旋转)")
            layout.label(text="3.绘制焊缝")
            layout.label(text="-在对象模式下单击 “绘制” 按钮")
            layout.label(text="-在模型的表面上绘制")
            layout.label(text="-在模型的表面上绘制")
            layout.label(text="-通过移动鼠标调整比例 (鼠标右键应用，设置为默认比例)")
            layout.label(text="-通过移动鼠标调整旋转 (鼠标右键应用，设置为默认旋转)")
            layout.label(text="4.配置文件编辑")
            layout.label(text="-选择焊接，")
            layout.label(text="-单击 '修改; 按钮")
            layout.label(text="-通过播放曲线小部件来调整焊接的轮廓")
            layout.label(text="-单击 “应用” 按钮接受结果")
    
        if self.prefs_tabs == 'options':
            box = layout.box()    
            row = box.row(align=True)
            row.label(text="侧边栏选项卡名称:")
            row.prop(self, "category", text="")
            row = box.row(align=True)
            row.label(text="性能:")
            row.prop(self, "performance", text="")   
            row = box.row(align=True) 
            row.label(text="交集解算器:")
            row.prop(self, "solver", text="")    